//
//  Detection.m
//  Resultados Video
//
//  Created by Pablo Roldán Ruz on 28/11/11.
//  Copyright (c) 2011 __MyCompanyName__. All rights reserved.
//

#import "Detection.h"


@implementation Detection 

@synthesize x;
@synthesize y;
@synthesize width;
@synthesize height;
@synthesize type;
@synthesize detected;

- (NSString *)description {
    return [NSString stringWithFormat:@"%d %d %d %d",self.x,self.y,self.width,self.height];
}


// Returns YES if the detection is true (i.e. intersection area >= 50% of the overlapping area)
- (BOOL)fitsWith:(Detection *)d {
    
    BOOL itFits = NO;
    
    /*
     * a1 +------------+ a2            
     *    | A          |               xa,ya --> coordinates of area A
     *    |   b1 +-----|------+ b2     xb,yb --> coordiantes of area B
     *    |      | B   |      | 
     *    |      |     |      |
     *    |      |     |      |        Each area is defined by 4 points (1,2,3,4)
     * a3 +------+-----+ a4   |
     *           |            |
     *        b3 +------------+ b4     */
    
    int xa1 = self.x;
    int ya1 = self.y;
    int xa2 = self.x + self.width;
    int ya3 = self.y + self.height;
    
    int xb1 = d.x;
    int yb1 = d.y;
    int xb2 = d.x + d.width;
    int yb3 = d.y + d.height;
      
    // Intersection area
    int xi1 = MAX(xa1, xb1);
    int yi1 = MAX(ya1, yb1);
    int xi2 = MIN(xa2, xb2);
    int yi3 = MIN(ya3, yb3);    
    
    if (((xa1 >= xb1 && xa1 <= xb2) || (xb1 >= xa1 && xb1 <= xa2)) && ((ya1 >= yb1 && ya1 <= yb3) || (yb1 >= ya1 && yb1 <= ya3))) {
    
        int widthi = xi2 - xi1;
        int heighti = yi3 - yi1;
    
        // Intersection area
        int supi = widthi * heighti;
        // Object A area
        int supa = self.width * self.height;
        // Object B area
        int supb = d.width * d.height;
        // Union area = SupA + SupB - SupIntersection
        int supu = supa + supb - supi;
        
        if (supi*100/supu >= 50) {
            itFits = YES;
        }
    }
    
    return itFits;    
}

@end
