//
//  ResultSet.m
//  Resultados Video
//
//  Created by Pablo Roldán Ruz on 28/11/11.
//  Copyright (c) 2011 __MyCompanyName__. All rights reserved.
//

#import "ResultSet.h"
#import "Frame.h"
#import "Detection.h"

@implementation ResultSet 

@synthesize frames;

// Loads the contents of the file fileName into the ResultSet.
- (id)initWithContentsOfFile:(NSString *)fileName {
    if (self = [super init]) {
        
        NSString *fileContent = [NSString stringWithContentsOfFile:fileName encoding:NSUTF8StringEncoding error:nil];
        
        // Frames are stored in a dictionary. The frame number is the key.
        frames = [[NSMutableDictionary dictionary] retain];
        
        NSArray *lines = [fileContent componentsSeparatedByCharactersInSet:[NSCharacterSet newlineCharacterSet]];
                
        for (NSString *line in lines) {
            NSArray *coordinates = [line componentsSeparatedByCharactersInSet:[NSCharacterSet characterSetWithCharactersInString:@" "]];
            
            // If it's a new frame, a new object is added to the array
            NSString *frameNumber = [coordinates objectAtIndex:0];
            
            if (![frames objectForKey:frameNumber]) {                
                Frame *newFrame = [[Frame alloc] init];
                newFrame.frameNumber = [frameNumber intValue];
                [frames setObject:newFrame forKey:frameNumber];
            }
            
            Detection *newDetection = [[[Detection alloc]init] retain];
            
            newDetection.x = [[coordinates objectAtIndex:1] intValue];
            newDetection.y = [[coordinates objectAtIndex:2] intValue];
            newDetection.width = [[coordinates objectAtIndex:3] intValue];
            newDetection.height = [[coordinates objectAtIndex:4] intValue];
            newDetection.detected = NO;
            
            [[[frames objectForKey:frameNumber] detections] addObject:newDetection];  
            
            [newDetection release];
        }
    }
    
    return self;
}

- (void)dealloc {
    [frames release];
    [super dealloc];
}


// Compares a ResultSet to another ResultSet. Those detections with detected = YES
// are considered true positives.
- (void)compareTo:(ResultSet *)set {
    
    for (NSString *frameNumber in self.frames) {
        
        Frame *f = [self.frames objectForKey:frameNumber];
        
        for (Detection *d in f.detections) {
            
            for (Detection *d2 in [[set.frames objectForKey:frameNumber] detections]) {
                
                if (d2.detected == NO && d.detected == NO) {                    
                    if ([d fitsWith:d2]) {
                        d2.detected = YES;
                        d.detected = YES;
                    }                    
                }                
            }            
        }        
    }
}


- (NSString *)description {
    
    NSMutableString *desc = [NSMutableString string];
    
    
    for (NSString *n in self.frames) {
        
        Frame *f = [self.frames objectForKey:n];
        
        [desc appendFormat:@"Frame %d: ",f.frameNumber];
        
        for (Detection *d in f.detections) {
            
            [desc appendFormat:@"%d %d %d %d",d.x,d.y,d.width,d.height];
            
            if (d.detected == YES) 
                [desc appendFormat:@" (DETECTED)\n"];
            else
                [desc appendFormat:@" (NOT DETECTED)\n"];
            
        }
        
    }
    
    return desc;
}


// returns the number of true positives
- (int)correctDetections {
    
    int n = 0;
    
    for (NSString *frameNumber in self.frames) {
        
        Frame *f = [self.frames objectForKey:frameNumber];
        
        for (Detection *d in f.detections) {
            if (d.detected == YES) {
                n++;
            }            
        }        
    }
    
    return n;    
}

// returns the number of false positives
- (int)incorrectDetections {
    
    int n = 0;
    
    for (NSString *frameNumber in self.frames) {
        
        Frame *f = [self.frames objectForKey:frameNumber];
        
        for (Detection *d in f.detections) {
            if (d.detected == NO) {
                n++;
            }            
        }        
    }
    
    return n;    
}


- (int)numberOfDetections {
    
    int n = 0;
    
    for (NSString *frameNumber in self.frames) {
        
        Frame *f = [self.frames objectForKey:frameNumber];
        
        for (Detection *d in f.detections) {
            n++;
        }        
    }
    
    return n;          
}




@end
