//
//  neon.c
//  VideoTestCV
//
//  Created by Pablo Roldán Ruz on 22/11/11.
//  Copyright (c) 2011 __MyCompanyName__. All rights reserved.
//


#include <stdio.h>
#include <arm_neon.h>
#include <stdio.h>

// NEON SIMD color to grayscale conversion
void neon_to_gray (cv::Mat &image, cv::Mat &grayMatImage);

// NEON SIMD color to grayscale conversion
static void neon_asm_convert(uint8_t * __restrict dest, uint8_t * __restrict src, int numPixels)
{
	__asm__ volatile("lsr          %2, %2, #3      \n"
					 "# build the three constants: \n"
					 "mov         r4, #28          \n" // Blue channel multiplier
					 "mov         r5, #151         \n" // Green channel multiplier
					 "mov         r6, #77          \n" // Red channel multiplier
					 "vdup.8      d4, r4           \n"
					 "vdup.8      d5, r5           \n"
					 "vdup.8      d6, r6           \n"
					 "0:						   \n"
					 "# load 8 pixels:             \n"
					 "vld4.8      {d0-d3}, [%1]!   \n"
					 "# do the weight average:     \n"
					 "vmull.u8    q7, d0, d4       \n"
					 "vmlal.u8    q7, d1, d5       \n"
					 "vmlal.u8    q7, d2, d6       \n"
					 "# shift and store:           \n"
					 "vshrn.u16   d7, q7, #8       \n" // Divide q3 by 256 and store in the d7
					 "vst1.8      {d7}, [%0]!      \n"
					 "subs        %2, %2, #1       \n" // Decrement iteration counter
					 "bne         0b            \n"    // Repeat unil iteration counter is not zero
					 :
					 : "r"(dest), "r"(src), "r"(numPixels)
					 : "r4", "r5", "r6"
					 );
}


// Converts a color image to a grayscale image using the NEON SIMD engine
void neon_to_gray (cv::Mat &image, cv::Mat &grayMatImage) {
    // Get information about the image
    uint8_t *baseAddress = (uint8_t *)image.data;
    size_t width = image.size().width; 
    size_t height = image.size().height;  
    
	// Create memory
	uint8_t* baseAddressGray = (uint8_t*) malloc(width*height);
	
	// Convert 
	neon_asm_convert(baseAddressGray, baseAddress, width*height); // 1 ms
    
	// Update data pointer
    grayMatImage.data = (uchar*)baseAddressGray;
}